from .parser import (
    MeteoAlarm,
    MeteoAlarmException,
    NoCountryError,
    NoRegionError,
    UnrecognizedCountryError,
    UnrecognizedRegionError,
    ParseError,
)

__version__ = "0.0.8"



import re
import feedparser

from ._resources import awl, awt, regions, countries


RE_TODAY = re.compile(r"(Today.*?)>Tomorrow<", re.I | re.M | re.S)
RE_TR = re.compile(r"<tr(.*?)</tr>", re.I | re.M | re.S)
RE_AWT = re.compile(r'alt="awt:(.*?) ', re.I | re.M | re.S)
RE_AWL = re.compile(r'level:(.*?)"', re.I | re.M | re.S)
RE_FROM = re.compile(r"From: </b><i>(.*?)</i><b>", re.I | re.M | re.S)
RE_UNTIL = re.compile(r"Until: </b><i>(.*?)</i>", re.I | re.M | re.S)
RE_MSG = re.compile(r"<td>(.*?)</td>", re.I | re.M | re.S)
RE_WS = re.compile(r"\s+", re.I | re.M | re.S)


class MeteoAlarmException(Exception):
    """Base class."""
    pass


class NoCountryError(MeteoAlarmException):
    pass


class NoRegionError(MeteoAlarmException):
    pass


class UnrecognizedCountryError(MeteoAlarmException):
    pass


class UnrecognizedRegionError(MeteoAlarmException):
    pass


class ParseError(MeteoAlarmException):
    pass


class MeteoAlarm:
    def __init__(self, country, region):
        if not country:
            raise NoCountryError()
        if country not in countries.keys():
            raise UnrecognizedCountryError()
        self._country = country
        if not region:
            raise NoRegionError()
        self._region = region
        if country in ("DE", "FR", "ES", "IT"):
            try:
                code = regions[country][region]
            except KeyError:
                raise UnrecognizedRegionError()
            url = "https://www.meteoalarm.eu/documents/rss/{iso}/{country}{code}.rss".format(
                iso=country.lower(), country=country.upper(), code=code
            )
        else:
            url = "https://www.meteoalarm.eu/documents/rss/{country}.rss".format(
                country=country.lower()
            )
        self._url = url

    @staticmethod
    def countries():
        return tuple(countries.keys())

    @staticmethod
    def awareness_levels():
        return tuple(awl.values())

    @staticmethod
    def awareness_types():
        return tuple(awt.values())

    def country_regions(self):
        return regions[self._country]

    def alerts(self):
        try:
            feed = feedparser.parse(self._url)
            alerts = [
                entry["description"]
                for entry in feed["entries"]
                if entry["title"] == self._region
            ]

            target = alerts[0] if alerts else ""
            if not target:
                return ()

            match = RE_TODAY.search(target)
            if not match:
                return ()

            buf = match.group()
            if not buf or "No special awareness required" in buf:
                return ()

            result = []
            rows = RE_TR.findall(buf)
            for i, row in enumerate(rows):

                if i % 2 == 0:
                    # get: awt, awl, from and until from rows 0, 2, 4, ...
                    atype = RE_AWT.search(row).group(1)
                    alevel = RE_AWL.search(row).group(1)
                    from_date = RE_FROM.search(row).group(1)
                    until_date = RE_UNTIL.search(row).group(1)
                else:
                    # get: msg from rows 1, 3, 5, ...
                    msg = RE_MSG.search(row).group(1).strip()
                    msg = msg.replace(".", ". ").strip()
                    msg = re.sub(r"\s+", " ", msg)

                    result.append(
                        {
                            "country": self._country.upper(),
                            "region": self._region,
                            "awareness_type": awt[atype],
                            "awareness_level": awl[alevel][0],
                            "from": from_date,
                            "until": until_date,
                            "message": msg,
                        },
                    )

        except Exception:
            raise ParseError

        return tuple(result)



# -*- coding: utf-8 -*-
# flake8: noqa
# isort:skip_file

# meteoalarm_rssapi -- an 'API' for meteoalarm.eu using the rss feeds.
# Copyright (C) 2021 Alexandre Lima Conde
# SPDX-License-Identifier: GPL-3.0-or-later

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime as dt
from setuptools import setup
from meteoalarm_rssapi import __version__

PROJECT_NAME = 'meteoalarm_rssapi'
PROJECT_PACKAGE_NAME = 'meteoalarm_rssapi'
PROJECT_LICENSE = 'GPL v3'
PROJECT_LICENSE_URL = 'https://github.com/xlcnd/meteoalarm-rssapi/blob/dev/LICENSE.txt'
PROJECT_AUTHOR = 'Alexandre Lima Conde'
PROJECT_COPYRIGHT = '{}, {}'.format(dt.now().year, PROJECT_AUTHOR)
PROJECT_URL = 'https://github.com/xlcnd/meteoalarm-rssapi'
PROJECT_EMAIL = 'xlcnd@outlook.com'
PROJECT_VERSION = __version__

PROJECT_GITHUB_USERNAME = 'xlcnd'
PROJECT_GITHUB_REPOSITORY = 'meteoalarm-rssapi'

GITHUB_PATH = '{}/{}'.format(PROJECT_GITHUB_USERNAME,
                             PROJECT_GITHUB_REPOSITORY)
GITHUB_URL = 'https://github.com/{}'.format(GITHUB_PATH)

DOWNLOAD_URL = '{}/archive/{}.zip'.format(GITHUB_URL, 'v' + PROJECT_VERSION)
PROJECT_URLS = {
    'Bug Reports': '{}/issues'.format(GITHUB_URL),
    'License': PROJECT_LICENSE_URL,
}

PYPI_URL = 'https://pypi.org/project/{}/'.format(PROJECT_PACKAGE_NAME)
PYPI_CLASSIFIERS = [
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Operating System :: OS Independent',
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Libraries :: Python Modules',
]

PACKAGES = [
    'meteoalarm_rssapi',
]

setup(
    name=PROJECT_PACKAGE_NAME,
    version=PROJECT_VERSION,
    url=PROJECT_URL,
    download_url=DOWNLOAD_URL,
    project_urls=PROJECT_URLS,
    author=PROJECT_AUTHOR,
    author_email=PROJECT_EMAIL,
    packages=PACKAGES,
    license=PROJECT_LICENSE,
    description='an "API" for meteoalarm.eu using the rss feeds.',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    keywords='meteoalarm',
    classifiers=PYPI_CLASSIFIERS,
    install_requires=['feedparser==6.0.2'],
)
